/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   Interrupt Handler Routine Mode 2
   siehe crt0.s fr Vektoren

*/
/////////////////////////////////////////////////////////

#include "main.h"

/*  Ring Puffer fr Uart
    
    Puffer wird von 0 beginnend an gefllt. Ist er voll wird prt = 0 gesetzt.    
    
    0....1....2....3....4....5....
    B0-> B1-> B2-> B3-> B4 ->B5->
    
    Es werden durch den RX Interrupt fortlaufend zeichen in den Buffer eingeschrieben.
    
    

*/

#define TRUE            1
#define FALSE           0
#define RX_BUF_LAST     80
#define TX_BUF_LAST     80

uint8_t rx_buffer[RX_BUF_LAST+1];   // Buffer fr Zeichenempfang
uint8_t rx_rd_ptr = 0;              // Read Pointer, nur durch Hauptprogramm lesen
uint8_t rx_wr_ptr = 0;              // Write Pointer, nur durch Interrupt schreiben


uint8_t rx_buf_full     = FALSE;    // Es sind Zeichen da
uint8_t rx_buf_crash    = FALSE;    // WR Zeiger hat RD berholt: Buffer wertlos

// uint8_t tx_buffer[TX_BUF_LAST+1];   // Sendepuffer
// uint8_t tx_ptr = 0;                 // Zeiger auf aktuelles Zeichen
// uint8_t rx_flag_full;               // Empfangspuffer voll


/////////////////////////////////////////////////////////
// NMI Handler siehe crt0.s Definition
// __critical __interrupt erzeugt ein retn am Ende statt reti

void nmi_vint(void) __critical __interrupt
{
     SEG7 = 0xff;
     __asm
         ld a,#0
         ld (0x2000),a
         jp 0x0000
    __endasm ;

}

// Lscht den Buffer und setzt Zeiger auf 0
void clear_rx_buffer()
{
    rx_wr_ptr = 0;
    rx_rd_ptr = 0;
    rx_buf_full = FALSE;
    rx_buf_crash = FALSE;
}

/* 
   Holt ein Zeichen aus dem Buffer fr das Hauptprogramm
   Darf nur von getchar aufgerufen werden, wenn rx_buf_full = TRUE ist!
   
*/   

char rx_read_char()
{
       static char ch;

       // Keine Zeichen da?
       if (!rx_buf_full)
            return(0);
       
       // RX Int aus, da Lesezugriff
       __asm
             di
       __endasm ;

       // Zeichen auslesen
       ch = rx_buffer[rx_rd_ptr++];

       // Read Pointer am Array Ende zurck setzen
       if (rx_rd_ptr > RX_BUF_LAST)
           rx_rd_ptr = 0;

       // Alle Zeichen gelesen? Dann liegen Pointer bereinander
       if (rx_rd_ptr == rx_wr_ptr)
           rx_buf_full    = FALSE;
             
       __asm
             ei
       __endasm ;
            
        return(ch);
}


/////////////////////////////////////////////////////////
// Handler der Mode 2 Interrupts der STI
// __interrupt erzeugt ein reti am Ende, Register werden gesichert
// __naked verhindert Erzeugung von Code zur Registersicherung auf dem Stack
// und erzeugt kein RETI

// Uart RX puffer (1 Byte) ist voll, es sind Zeichen da!
void int_sti_receive_buffer_full(void)  __interrupt
{
    static char zeichen;
    static char crash = 0;

    zeichen = STI_UDR;      
    PIO8255_PORT_A = ~zeichen;

    // Ringpuffer beschreiben    
    rx_buffer[rx_wr_ptr] = zeichen;

    // wr Zeiger auf letztem Element?
    if (++rx_wr_ptr > RX_BUF_LAST) 
        rx_wr_ptr = 0;                 // Zeiger zurck setzen
    
    // WR==RD oder WR > RD: Buffer Crash
    rx_buf_crash = (rx_wr_ptr==rx_rd_ptr);
         
    rx_buf_full    = TRUE;             // Meldung an Hauptprogramm: Zeichen ist da
        
    __asm
         ei
    __endasm ;
}

// Interrupt des Timers A
// Zyklischer Interrupt fr Aktualisierungen von Hardware
void int_sti_timer_a(void) __interrupt
{
    static char n;

    PIO8255_PORT_A = ~(n++);

    __asm
         ei
    __endasm ;
}



////////////////////////////////////////
// ------- Unbenutzte Ints -------------
////////////////////////////////////////

// AD Wandler Interrupt ber STI
void int_sti_gpi_0(void) __interrupt __naked
{


    __asm
         ei
         reti
    __endasm ;
}

// Interrupt des Timers B
void int_sti_timer_b(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

// Interrupt des Timers C
void int_sti_timer_c(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

// Uart Sendepuffer ist leer
void int_sti_transmit_buffer_empty(void)  __interrupt
{
    __asm
         ei
    __endasm ;
}

void int_sti_timer_d(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

// Unbenutzte Interrupts, jedoch mssen Funktionen bereitstehen

void int_sti_gpi_1(void)  __interrupt __naked
{

    __asm
         ei
         reti
    __endasm ;
}
void int_sti_gpi_2(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}
void int_sti_gpi_3(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_gpi_4(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_gpi_5(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_gpi_6(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_gpi_7(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_transmit_error(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}

void int_sti_receive_error(void)  __interrupt __naked
{
    __asm
         ei
         reti
    __endasm ;
}






